@@ -45,13 +45,13 @@
 #include <nm-device-wifi.h>
 #include <nm-gsm-device.h>
 #include <nm-cdma-device.h>
+#include <nm-device-bt.h>
 #include <nm-utils.h>
 #include <nm-connection.h>
 #include <nm-vpn-connection.h>
 #include <nm-setting-connection.h>
 #include <nm-setting-vpn.h>
 #include <nm-active-connection.h>
-#include <nm-dbus-settings-system.h>
 #include <nm-setting-wireless.h>
 
 #include <glade/glade.h>
@@ -64,6 +64,7 @@
 #include "applet-device-wifi.h"
 #include "applet-device-gsm.h"
 #include "applet-device-cdma.h"
+#include "applet-device-bt.h"
 #include "applet-dialogs.h"
 #include "vpn-password-dialog.h"
 #include "applet-dbus-manager.h"
@@ -176,35 +177,17 @@ applet_get_default_active_connection (NMApplet *applet, NMDevice **device)
 	return default_ac;
 }
 
-static void
-exported_connection_to_connection (gpointer data, gpointer user_data)
-{
-	GSList **list = (GSList **) user_data;
-
-	*list = g_slist_prepend (*list, nm_exported_connection_get_connection (NM_EXPORTED_CONNECTION (data)));
-}
-
-NMSettings *
+NMSettingsInterface *
 applet_get_settings (NMApplet *applet)
 {
-	return (NMSettings *) applet->gconf_settings;
+	return NM_SETTINGS_INTERFACE (applet->gconf_settings);
 }
 
 GSList *
 applet_get_all_connections (NMApplet *applet)
 {
-	GSList *list;
-	GSList *connections = NULL;
-
-	list = nm_settings_list_connections (NM_SETTINGS (applet->dbus_settings));
-	g_slist_foreach (list, exported_connection_to_connection, &connections);
-	g_slist_free (list);
-
-	list = nm_settings_list_connections (NM_SETTINGS (applet->gconf_settings));
-	g_slist_foreach (list, exported_connection_to_connection, &connections);
-	g_slist_free (list);
-
-	return connections;
+	return g_slist_concat (nm_settings_interface_list_connections (NM_SETTINGS_INTERFACE (applet->system_settings)),
+	                       nm_settings_interface_list_connections (NM_SETTINGS_INTERFACE (applet->gconf_settings)));
 }
 
 static NMConnection *
@@ -254,9 +237,11 @@ applet_get_active_for_connection (NMApplet *applet, NMConnection *connection)
 	active_list = nm_client_get_active_connections (applet->nm_client);
 	for (i = 0; active_list && (i < active_list->len); i++) {
 		NMActiveConnection *active = NM_ACTIVE_CONNECTION (g_ptr_array_index (active_list, i));
+		const char *active_cpath = nm_active_connection_get_connection (active);
 
 		if (   (nm_active_connection_get_scope (active) == scope)
-		    && !strcmp (nm_active_connection_get_connection (active), cpath))
+		    && active_cpath
+		    && !strcmp (active_cpath, cpath))
 			return active;
 	}
 	return NULL;
@@ -276,6 +261,8 @@ get_device_class (NMDevice *device, NMApplet *applet)
 		return applet->gsm_class;
 	else if (NM_IS_CDMA_DEVICE (device))
 		return applet->cdma_class;
+	else if (NM_IS_DEVICE_BT (device))
+		return applet->bt_class;
 	else
 		g_message ("%s: Unknown device type '%s'", __func__, G_OBJECT_TYPE_NAME (device));
 	return NULL;
@@ -355,6 +342,7 @@ applet_menu_item_activate_helper_part2 (NMConnection *connection,
 			return;
 		}
 		g_object_unref (connection);
+		connection = NM_CONNECTION (exported);
 	}
 
 	g_assert (connection);
@@ -372,6 +360,27 @@ applet_menu_item_activate_helper_part2 (NMConnection *connection,
 	applet_item_activate_info_destroy (info);
 }
 
+static void
+disconnect_cb (NMDevice *device, GError *error, gpointer user_data)
+{
+	if (error) {
+		g_warning ("%s: device disconnect failed: (%d) %s",
+		           __func__,
+		           error ? error->code : -1,
+		           error && error->message ? error->message : "(unknown)");
+	}
+}
+
+void
+applet_menu_item_disconnect_helper (NMDevice *device,
+                                    NMApplet *applet)
+{
+	g_return_if_fail (NM_IS_DEVICE (device));
+
+	nm_device_disconnect (device, disconnect_cb, NULL);
+}
+
+
 void
 applet_menu_item_activate_helper (NMDevice *device,
                                   NMConnection *connection,
@@ -410,6 +419,188 @@ applet_menu_item_activate_helper (NMDevice *device,
 	}
 }
 
+void
+applet_menu_item_add_complex_separator_helper (GtkWidget *menu,
+                                               NMApplet *applet,
+                                               const gchar* label,
+                                               int pos)
+{
+	GtkWidget *menu_item = gtk_image_menu_item_new ();
+	GtkWidget *box = gtk_hbox_new (FALSE, 0);
+	GtkWidget *xlabel = NULL;
+
+	if (label) {
+		xlabel = gtk_label_new (NULL);
+		gtk_label_set_markup (GTK_LABEL (xlabel), label);
+
+		gtk_box_pack_start (GTK_BOX (box), gtk_hseparator_new (), TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (box), xlabel, FALSE, FALSE, 2);
+	}
+
+	gtk_box_pack_start (GTK_BOX (box), gtk_hseparator_new (), TRUE, TRUE, 0);
+
+	g_object_set (G_OBJECT (menu_item),
+	              "child", box,
+	              "sensitive", FALSE,
+	              NULL);
+	if (pos < 0)
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+	else
+		gtk_menu_shell_insert (GTK_MENU_SHELL (menu), menu_item, pos);
+	return;
+}
+
+GtkWidget *
+applet_new_menu_item_helper (NMConnection *connection,
+                             NMConnection *active,
+                             gboolean add_active)
+{
+	GtkWidget *item;
+	NMSettingConnection *s_con;
+	char *markup;
+	GtkWidget *label;
+
+	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+	item = gtk_image_menu_item_new_with_label ("");
+	if (add_active && (active == connection)) {
+		/* Pure evil */
+		label = gtk_bin_get_child (GTK_BIN (item));
+		gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+		markup = g_markup_printf_escaped ("<b>%s</b>", nm_setting_connection_get_id (s_con));
+		gtk_label_set_markup (GTK_LABEL (label), markup);
+		g_free (markup);
+	} else
+		gtk_menu_item_set_label (GTK_MENU_ITEM (item), nm_setting_connection_get_id (s_con));
+
+	gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (item), TRUE);
+	return item;
+}
+
+#define TITLE_TEXT_R ((double) 0x5e / 255.0 )
+#define TITLE_TEXT_G ((double) 0x5e / 255.0 )
+#define TITLE_TEXT_B ((double) 0x5e / 255.0 )
+
+static gboolean
+menu_title_item_expose (GtkWidget *widget, GdkEventExpose *event)
+{
+	GtkStyle *style;
+	GtkWidget *label;
+	PangoFontDescription *desc;
+	cairo_t *cr;
+	GtkImageMenuItem *image_menu_item;
+	GtkImage *icon;
+	GdkPixbuf *pixbuf;
+	PangoLayout *layout;
+	int width = 0, height = 0, owidth, oheight;
+	gdouble extraheight = 0, extrawidth = 0;
+	const char *text;
+	gdouble xpadding = 10.0;
+	gdouble ypadding = 5.0;
+	gdouble postpadding = 0.0;
+
+	label = gtk_bin_get_child (GTK_BIN (widget));
+
+	cr = gdk_cairo_create (widget->window);
+
+	/* The drawing area we get is the whole menu; clip the drawing to the
+	 * event area, which should just be our menu item.
+	 */
+	cairo_rectangle (cr,
+	                 event->area.x, event->area.y,
+	                 event->area.width, event->area.height);
+	cairo_clip (cr);
+
+	/* We also need to reposition the cairo context so that (0, 0) is the
+	 * top-left of where we're supposed to start drawing.
+	 */
+	cairo_translate (cr, widget->allocation.x, widget->allocation.y);
+
+	image_menu_item = GTK_IS_IMAGE_MENU_ITEM (widget) ? GTK_IMAGE_MENU_ITEM (widget) : NULL;
+
+	icon = image_menu_item ? GTK_IMAGE (gtk_image_menu_item_get_image (image_menu_item)) : NULL;
+	pixbuf = icon ? gtk_image_get_pixbuf (icon) : NULL;
+
+	text = gtk_label_get_text (GTK_LABEL (label));
+
+	layout = pango_cairo_create_layout (cr);
+	style = gtk_widget_get_style (widget);
+	desc = pango_font_description_copy (style->font_desc);
+	pango_font_description_set_variant (desc, PANGO_VARIANT_SMALL_CAPS);
+	pango_font_description_set_weight (desc, PANGO_WEIGHT_SEMIBOLD);
+	pango_layout_set_font_description (layout, desc);
+	pango_layout_set_text (layout, text, -1);
+	pango_cairo_update_layout (cr, layout);
+	pango_layout_get_size (layout, &owidth, &oheight);
+	width = owidth / PANGO_SCALE;
+	height += oheight / PANGO_SCALE;
+
+	if (pixbuf) {
+		gdouble w = gdk_pixbuf_get_width (pixbuf);
+		gdouble h = gdk_pixbuf_get_height (pixbuf);
+
+		if (width < w)
+			width = w;
+		if (height < h) {
+			extraheight = (h - height) / 2.0;
+			height = h;
+		}
+		extrawidth = gdk_pixbuf_get_width (pixbuf) + 5.0 + 5.0;
+	}
+	width += extrawidth;
+
+	cairo_save (cr);
+
+	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.0);
+	cairo_rectangle (cr, 0, 0,
+	                 (double) (width + 2 * xpadding),
+	                 (double) (height + ypadding + postpadding));
+	cairo_fill (cr);
+
+	/* now the in-padding content */
+	cairo_translate (cr, xpadding , ypadding);
+	if (pixbuf) {
+		gdouble x = 0.0;
+		gdouble y = height / 2.0 - gdk_pixbuf_get_height (pixbuf) / 2.0;
+		gdouble w = gdk_pixbuf_get_width (pixbuf);
+		gdouble h = gdk_pixbuf_get_height (pixbuf);
+
+		gdk_cairo_set_source_pixbuf (cr, pixbuf, x, y);
+		cairo_rectangle (cr, x, y, w, h);
+		cairo_fill (cr);
+	}
+
+	cairo_set_source_rgb (cr, TITLE_TEXT_R, TITLE_TEXT_G, TITLE_TEXT_B);
+	cairo_move_to (cr, extrawidth, extraheight);
+	pango_cairo_show_layout (cr, layout);
+
+	cairo_restore(cr);
+
+	pango_font_description_free (desc);
+	g_object_unref (layout);
+	cairo_destroy (cr);
+
+	gtk_widget_set_size_request (widget, width + 2 * xpadding, height + ypadding + postpadding);
+	return TRUE;
+}
+
+
+GtkWidget *
+applet_menu_item_create_device_item_helper (NMDevice *device,
+                                            NMApplet *applet,
+                                            const gchar *text)
+{
+	GtkWidget *item = gtk_image_menu_item_new_with_mnemonic (text);
+	NMADeviceClass *klass = get_device_class (device, applet);
+	GdkPixbuf *pixbuf = klass->get_device_icon ? klass->get_device_icon (device, applet) : NULL;
+	GtkWidget *image = pixbuf ? gtk_image_new_from_pixbuf (pixbuf) : NULL;
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+
+	gtk_widget_set_sensitive (item, FALSE);
+
+	g_signal_connect (item, "expose-event", G_CALLBACK (menu_title_item_expose), NULL);
+	return item;
+}
+
 static void
 applet_clear_notify (NMApplet *applet)
 {
@@ -571,20 +762,32 @@ applet_is_any_vpn_activating (NMApplet *applet)
 	}
 	return FALSE;
 }
+static void
+save_timestamp_cb (NMSettingsConnectionInterface *connection,
+                   GError *error,
+                   gpointer user_data)
+{
+	if (error) {
+		g_warning ("Error saving connection %s timestamp: (%d) %s",
+		           nm_connection_get_path (NM_CONNECTION (connection)),
+		           error->code,
+		           error->message);
+	}
+}
+
 static void
 update_connection_timestamp (NMActiveConnection *active,
                              NMConnection *connection,
                              NMApplet *applet)
 {
-	NMAGConfConnection *gconf_connection;
-	const char *path;
+	NMSettingsConnectionInterface *gconf_connection;
 	NMSettingConnection *s_con;
 
 	if (nm_active_connection_get_scope (active) != NM_CONNECTION_SCOPE_USER)
 		return;
 
-	path = nm_connection_get_path (connection);
-	gconf_connection = nma_gconf_settings_get_by_dbus_path (applet->gconf_settings, path);
+	gconf_connection = nm_settings_interface_get_connection_by_path (NM_SETTINGS_INTERFACE (applet->gconf_settings),
+	                                                                 nm_connection_get_path (connection));
 	if (!gconf_connection)
 		return;
 
@@ -592,7 +795,7 @@ update_connection_timestamp (NMActiveConnection *active,
 	g_assert (s_con);
 
 	g_object_set (s_con, NM_SETTING_CONNECTION_TIMESTAMP, (guint64) time (NULL), NULL);
-	nma_gconf_connection_save (gconf_connection);
+	nm_settings_connection_interface_update (gconf_connection, save_timestamp_cb, NULL);
 }
 
 static char *
@@ -981,14 +1184,23 @@ sort_devices (gconstpointer a, gconstpointer b)
 		return -1;
 	if (aa_type == NM_TYPE_DEVICE_ETHERNET && bb_type == NM_TYPE_CDMA_DEVICE)
 		return -1;
+	if (aa_type == NM_TYPE_DEVICE_ETHERNET && bb_type == NM_TYPE_DEVICE_BT)
+		return -1;
 
 	if (aa_type == NM_TYPE_GSM_DEVICE && bb_type == NM_TYPE_CDMA_DEVICE)
 		return -1;
 	if (aa_type == NM_TYPE_GSM_DEVICE && bb_type == NM_TYPE_DEVICE_WIFI)
 		return -1;
+	if (aa_type == NM_TYPE_GSM_DEVICE && bb_type == NM_TYPE_DEVICE_BT)
+		return -1;
 
 	if (aa_type == NM_TYPE_CDMA_DEVICE && bb_type == NM_TYPE_DEVICE_WIFI)
 		return -1;
+	if (aa_type == NM_TYPE_CDMA_DEVICE && bb_type == NM_TYPE_DEVICE_BT)
+		return -1;
+
+	if (aa_type == NM_TYPE_DEVICE_BT && bb_type == NM_TYPE_DEVICE_WIFI)
+		return -1;
 
 	return 1;
 }
@@ -1021,6 +1233,8 @@ applet_find_active_connection_for_device (NMDevice *device,
 
 	active_connections = nm_client_get_active_connections (applet->nm_client);
 	for (i = 0; active_connections && (i < active_connections->len); i++) {
+		NMSettingsConnectionInterface *tmp;
+		NMSettingsInterface *settings = NULL;
 		NMActiveConnection *active;
 		const char *service_name;
 		const char *connection_path;
@@ -1037,35 +1251,69 @@ applet_find_active_connection_for_device (NMDevice *device,
 		if (!nm_g_ptr_array_contains (devices, device))
 			continue;
 
-		if (!strcmp (service_name, NM_DBUS_SERVICE_SYSTEM_SETTINGS)) {
-			NMDBusConnection *tmp;
+		if (!strcmp (service_name, NM_DBUS_SERVICE_SYSTEM_SETTINGS))
+			settings = NM_SETTINGS_INTERFACE (applet->system_settings);
+		else if (!strcmp (service_name, NM_DBUS_SERVICE_USER_SETTINGS))
+			settings = NM_SETTINGS_INTERFACE (applet->gconf_settings);
+		else
+			g_assert_not_reached ();
 
-			tmp = nm_dbus_settings_get_connection_by_path (applet->dbus_settings, connection_path);
-			if (tmp) {
-				connection = nm_exported_connection_get_connection (NM_EXPORTED_CONNECTION (tmp));
-				if (out_active)
-					*out_active = active;
-				break;
-			}
-		} else if (!strcmp (service_name, NM_DBUS_SERVICE_USER_SETTINGS)) {
-			NMAGConfConnection *tmp;
-
-			tmp = nma_gconf_settings_get_by_dbus_path (applet->gconf_settings, connection_path);
-			if (tmp) {
-				connection = nm_exported_connection_get_connection (NM_EXPORTED_CONNECTION (tmp));
-				if (out_active)
-					*out_active = active;
-				break;
-			}
+		tmp = nm_settings_interface_get_connection_by_path (settings, connection_path);
+		if (tmp) {
+			connection = NM_CONNECTION (tmp);
+			if (out_active)
+				*out_active = active;
+			break;
 		}
 	}
 
 	return connection;
 }
 
+gboolean
+nma_menu_device_check_unusable (NMDevice *device)
+{
+	switch (nm_device_get_state (device)) {
+	case NM_DEVICE_STATE_UNKNOWN:
+	case NM_DEVICE_STATE_UNAVAILABLE:
+	case NM_DEVICE_STATE_UNMANAGED:
+		return TRUE;
+	default:
+		break;
+	}
+	return FALSE;
+}
+
+
+struct AppletDeviceMenuInfo {
+	NMDevice *device;
+	NMApplet *applet;
+};
+
+static void
+applet_device_info_destroy (struct AppletDeviceMenuInfo *info)
+{
+	g_return_if_fail (info != NULL);
+
+	if (info->device)
+		g_object_unref (info->device);
+	memset (info, 0, sizeof (struct AppletDeviceMenuInfo));
+	g_free (info);
+}
+
+static void
+applet_device_disconnect_db (GtkMenuItem *item, gpointer user_data)
+{
+	struct AppletDeviceMenuInfo *info = user_data;
+
+	applet_menu_item_disconnect_helper (info->device,
+	                                    info->applet);
+}
+
 GtkWidget *
-nma_menu_device_check_unusable (NMDevice *device,
-                                const char *unavailable_msg)
+nma_menu_device_get_menu_item (NMDevice *device,
+                               NMApplet *applet,
+                               const char *unavailable_msg)
 {
 	GtkWidget *item = NULL;
 	gboolean managed = TRUE;
@@ -1077,20 +1325,42 @@ nma_menu_device_check_unusable (NMDevice *device,
 	case NM_DEVICE_STATE_UNKNOWN:
 	case NM_DEVICE_STATE_UNAVAILABLE:
 		item = gtk_menu_item_new_with_label (unavailable_msg);
+		gtk_widget_set_sensitive (item, FALSE);
+		break;
+	case NM_DEVICE_STATE_DISCONNECTED:
+		unavailable_msg = _("disconnected");
+		item = gtk_menu_item_new_with_label (unavailable_msg);
+		gtk_widget_set_sensitive (item, FALSE);
 		break;
 	case NM_DEVICE_STATE_UNMANAGED:
 		managed = FALSE;
 		break;
+	case NM_DEVICE_STATE_PREPARE:
+	case NM_DEVICE_STATE_CONFIG:
+	case NM_DEVICE_STATE_NEED_AUTH:
+	case NM_DEVICE_STATE_IP_CONFIG:
+	case NM_DEVICE_STATE_ACTIVATED:
+	{
+		struct AppletDeviceMenuInfo *info = g_new0 (struct AppletDeviceMenuInfo, 1);
+		info->device = g_object_ref (device);
+		info->applet = applet;
+		item = gtk_menu_item_new_with_label (_("Disconnect"));
+		g_signal_connect_data (item, "activate",
+		                       G_CALLBACK (applet_device_disconnect_db),
+		                       info,
+		                       (GClosureNotify) applet_device_info_destroy, 0);
+		gtk_widget_set_sensitive (item, TRUE);
+		break;
+	}
 	default:
 		managed = nm_device_get_managed (device);
 		break;
 	}
 
-	if (!managed)
+	if (!managed) {
 		item = gtk_menu_item_new_with_label (_("device not managed"));
-
-	if (item)
 		gtk_widget_set_sensitive (item, FALSE);
+	}
 
 	return item;
 }
@@ -1104,6 +1374,7 @@ nma_menu_add_devices (GtkWidget *menu, NMApplet *applet)
 	gint n_usable_wifi_devices = 0;
 	gint n_wired_devices = 0;
 	gint n_mb_devices = 0;
+	gint n_bt_devices = 0;
 	int i;
 
 	temp = nm_client_get_devices (applet->nm_client);
@@ -1128,9 +1399,11 @@ nma_menu_add_devices (GtkWidget *menu, NMApplet *applet)
 			n_wired_devices++;
 		else if (NM_IS_CDMA_DEVICE (device) || NM_IS_GSM_DEVICE (device))
 			n_mb_devices++;
+		else if (NM_IS_DEVICE_BT (device))
+			n_bt_devices++;
 	}
 
-	if (!n_wired_devices && !n_wifi_devices && !n_mb_devices) {
+	if (!n_wired_devices && !n_wifi_devices && !n_mb_devices && !n_bt_devices) {
 		nma_menu_add_text_item (menu, _("No network devices available"));
 		goto out;
 	}
@@ -1235,11 +1508,12 @@ nma_menu_add_vpn_submenu (GtkWidget *menu, NMApplet *applet)
 		NMConnection *connection = NM_CONNECTION (iter->data);
 		NMActiveConnection *active;
 		const char *name;
+		GtkWidget *image;
 
 		name = get_connection_id (connection);
 
-		item = GTK_MENU_ITEM (gtk_check_menu_item_new_with_label (name));
-		gtk_check_menu_item_set_draw_as_radio (GTK_CHECK_MENU_ITEM (item), TRUE);
+		item = GTK_MENU_ITEM (gtk_image_menu_item_new_with_label (name));
+		gtk_image_menu_item_set_always_show_image(GTK_IMAGE_MENU_ITEM(item), TRUE);
 
 		/* If no VPN connections are active, draw all menu items enabled. If
 		 * >= 1 VPN connections are active, only the active VPN menu item is
@@ -1254,8 +1528,10 @@ nma_menu_add_vpn_submenu (GtkWidget *menu, NMApplet *applet)
 		else
 			gtk_widget_set_sensitive (GTK_WIDGET (item), FALSE);
 
-		if (active)
-			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), TRUE);
+		if (active) {
+			image = gtk_image_new_from_stock (GTK_STOCK_CONNECT, GTK_ICON_SIZE_MENU);
+			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+		}
 
 		g_object_set_data_full (G_OBJECT (item), "connection", 
 						    g_object_ref (connection),
@@ -1294,6 +1570,16 @@ nma_set_wireless_enabled_cb (GtkWidget *widget, NMApplet *applet)
 	nm_client_wireless_set_enabled (applet->nm_client, state);
 }
 
+static void
+nma_set_wwan_enabled_cb (GtkWidget *widget, NMApplet *applet)
+{
+	gboolean state;
+
+	g_return_if_fail (applet != NULL);
+
+	state = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget));
+	nm_client_wwan_set_enabled (applet->nm_client, state);
+}
 
 static void
 nma_set_networking_enabled_cb (GtkWidget *widget, NMApplet *applet)
@@ -1440,7 +1726,9 @@ nma_context_menu_update (NMApplet *applet)
 {
 	NMState state;
 	gboolean have_wireless = FALSE;
+	gboolean have_wwan = FALSE;
 	gboolean wireless_hw_enabled;
+	gboolean wwan_hw_enabled;
 	gboolean notifications_enabled = TRUE;
 
 	state = nm_client_get_state (applet->nm_client);
@@ -1471,6 +1759,18 @@ nma_context_menu_update (NMApplet *applet)
 	gtk_widget_set_sensitive (GTK_WIDGET (applet->wifi_enabled_item),
 	                          wireless_hw_enabled);
 
+	/* Enabled Mobile Broadband */
+	g_signal_handler_block (G_OBJECT (applet->wwan_enabled_item),
+	                        applet->wwan_enabled_toggled_id);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (applet->wwan_enabled_item),
+	                                nm_client_wwan_get_enabled (applet->nm_client));
+	g_signal_handler_unblock (G_OBJECT (applet->wwan_enabled_item),
+	                          applet->wwan_enabled_toggled_id);
+
+	wwan_hw_enabled = nm_client_wwan_hardware_get_enabled (applet->nm_client);
+	gtk_widget_set_sensitive (GTK_WIDGET (applet->wwan_enabled_item),
+	                          wwan_hw_enabled);
+
 	/* Enabled notifications */
 	g_signal_handler_block (G_OBJECT (applet->notifications_enabled_item),
 	                        applet->notifications_enabled_toggled_id);
@@ -1489,10 +1789,12 @@ nma_context_menu_update (NMApplet *applet)
 
 		devices = nm_client_get_devices (applet->nm_client);
 		for (i = 0; devices && (i < devices->len); i++) {
-			if (NM_IS_DEVICE_WIFI (g_ptr_array_index (devices, i))) {
+			NMDevice *candidate = g_ptr_array_index (devices, i);
+
+			if (NM_IS_DEVICE_WIFI (candidate))
 				have_wireless = TRUE;
-				break;
-			}
+			else if (NM_IS_SERIAL_DEVICE (candidate))
+				have_wwan = TRUE;
 		}
 	}
 
@@ -1500,6 +1802,11 @@ nma_context_menu_update (NMApplet *applet)
 		gtk_widget_show_all (applet->wifi_enabled_item);
 	else
 		gtk_widget_hide (applet->wifi_enabled_item);
+
+	if (have_wwan)
+		gtk_widget_show_all (applet->wwan_enabled_item);
+	else
+		gtk_widget_hide (applet->wwan_enabled_item);
 }
 
 static void
@@ -1568,6 +1875,15 @@ static GtkWidget *nma_context_menu_create (NMApplet *applet)
 	applet->wifi_enabled_toggled_id = id;
 	gtk_menu_shell_append (menu, applet->wifi_enabled_item);
 
+	/* 'Enable Mobile Broadband' item */
+	applet->wwan_enabled_item = gtk_check_menu_item_new_with_mnemonic (_("Enable _Mobile Broadband"));
+	id = g_signal_connect (applet->wwan_enabled_item,
+	                       "toggled",
+	                       G_CALLBACK (nma_set_wwan_enabled_cb),
+	                       applet);
+	applet->wwan_enabled_toggled_id = id;
+	gtk_menu_shell_append (menu, applet->wwan_enabled_item);
+
 	nma_menu_add_separator_item (GTK_WIDGET (menu));
 
 	/* Toggle notifications item */
@@ -1674,7 +1990,7 @@ foo_set_icon (NMApplet *applet, GdkPixbuf *pixbuf, guint32 layer)
 }
 
 
-NMAGConfConnection *
+NMSettingsConnectionInterface *
 applet_get_exported_connection_for_device (NMDevice *device, NMApplet *applet)
 {
 	const GPtrArray *active_connections;
@@ -1683,7 +1999,7 @@ applet_get_exported_connection_for_device (NMDevice *device, NMApplet *applet)
 	active_connections = nm_client_get_active_connections (applet->nm_client);
 	for (i = 0; active_connections && (i < active_connections->len); i++) {
 		NMActiveConnection *active;
-		NMAGConfConnection *gconf_connection;
+		NMSettingsConnectionInterface *connection;
 		const char *service_name;
 		const char *connection_path;
 		const GPtrArray *devices;
@@ -1699,11 +2015,9 @@ applet_get_exported_connection_for_device (NMDevice *device, NMApplet *applet)
 		if (!nm_g_ptr_array_contains (devices, device))
 			continue;
 
-		gconf_connection = nma_gconf_settings_get_by_dbus_path (applet->gconf_settings, connection_path);
-		if (!gconf_connection || !nm_exported_connection_get_connection (NM_EXPORTED_CONNECTION (gconf_connection)))
-			continue;
-
-		return gconf_connection;
+		connection = nm_settings_interface_get_connection_by_path (NM_SETTINGS_INTERFACE (applet->gconf_settings), connection_path);
+		if (connection)
+			return connection;
 	}
 	return NULL;
 }
@@ -2161,14 +2475,14 @@ applet_schedule_update_icon (NMApplet *applet)
 }
 
 static NMDevice *
-find_active_device (NMAGConfConnection *exported,
+find_active_device (NMAGConfConnection *connection,
                     NMApplet *applet,
                     NMActiveConnection **out_active_connection)
 {
 	const GPtrArray *active_connections;
 	int i;
 
-	g_return_val_if_fail (exported != NULL, NULL);
+	g_return_val_if_fail (connection != NULL, NULL);
 	g_return_val_if_fail (applet != NULL, NULL);
 	g_return_val_if_fail (out_active_connection != NULL, NULL);
 	g_return_val_if_fail (*out_active_connection == NULL, NULL);
@@ -2179,20 +2493,29 @@ find_active_device (NMAGConfConnection *exported,
 	active_connections = nm_client_get_active_connections (applet->nm_client);
 	for (i = 0; active_connections && (i < active_connections->len); i++) {
 		NMActiveConnection *active;
-		NMConnection *connection;
 		const char *service_name;
 		const char *connection_path;
 		const GPtrArray *devices;
 
 		active = NM_ACTIVE_CONNECTION (g_ptr_array_index (active_connections, i));
 		service_name = nm_active_connection_get_service_name (active);
+		if (!service_name) {
+			/* Shouldn't happen; but we shouldn't crash either */
+			g_warning ("%s: couldn't get service name for active connection!", __func__);
+			continue;
+		}
+
 		if (strcmp (service_name, NM_DBUS_SERVICE_USER_SETTINGS))
 			continue;
 
 		connection_path = nm_active_connection_get_connection (active);
+		if (!connection_path) {
+			/* Shouldn't happen; but we shouldn't crash either */
+			g_warning ("%s: couldn't get connection path for active connection!", __func__);
+			continue;
+		}
 
-		connection = nm_exported_connection_get_connection (NM_EXPORTED_CONNECTION (exported));
-		if (!strcmp (connection_path, nm_connection_get_path (connection))) {
+		if (!strcmp (connection_path, nm_connection_get_path (NM_CONNECTION (connection)))) {
 			devices = nm_active_connection_get_devices (active);
 			if (devices)
 				*out_active_connection = active;
@@ -2205,37 +2528,36 @@ find_active_device (NMAGConfConnection *exported,
 
 static void
 applet_settings_new_secrets_requested_cb (NMAGConfSettings *settings,
-                                          NMAGConfConnection *exported,
+                                          NMAGConfConnection *connection,
                                           const char *setting_name,
                                           const char **hints,
                                           gboolean ask_user,
-                                          DBusGMethodInvocation *context,
+                                          NMANewSecretsRequestedFunc callback,
+                                          gpointer callback_data,
                                           gpointer user_data)
 {
 	NMApplet *applet = NM_APPLET (user_data);
 	NMActiveConnection *active_connection = NULL;
-	NMConnection *connection;
 	NMSettingConnection *s_con;
 	NMDevice *device;
 	NMADeviceClass *dclass;
 	GError *error = NULL;
 
-	connection = nm_exported_connection_get_connection (NM_EXPORTED_CONNECTION (exported));
-	g_return_if_fail (connection != NULL);
-
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+	s_con = (NMSettingConnection *) nm_connection_get_setting (NM_CONNECTION (connection), NM_TYPE_SETTING_CONNECTION);
 	g_return_if_fail (s_con != NULL);
 
 	/* VPN secrets get handled a bit differently */
 	if (!strcmp (nm_setting_connection_get_connection_type (s_con), NM_SETTING_VPN_SETTING_NAME)) {
-		nma_vpn_request_password (NM_EXPORTED_CONNECTION (exported), ask_user, context);
+		nma_vpn_request_password (NM_SETTINGS_CONNECTION_INTERFACE (connection), ask_user, callback, callback_data);
 		return;
 	}
 
 	/* Find the active device for this connection */
-	device = find_active_device (exported, applet, &active_connection);
+	device = find_active_device (connection, applet, &active_connection);
 	if (!device || !active_connection) {
-		g_set_error (&error, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_INTERNAL_ERROR,
+		g_set_error (&error,
+		             NM_SETTINGS_INTERFACE_ERROR,
+		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
 		             "%s.%d (%s): couldn't find details for connection",
 		             __FILE__, __LINE__, __func__);
 		goto error;
@@ -2243,29 +2565,35 @@ applet_settings_new_secrets_requested_cb (NMAGConfSettings *settings,
 
 	dclass = get_device_class (device, applet);
 	if (!dclass) {
-		g_set_error (&error, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_INTERNAL_ERROR,
+		g_set_error (&error,
+		             NM_SETTINGS_INTERFACE_ERROR,
+		             NM_SETTINGS_INTERFACE_ERROR_INTERNAL_ERROR,
 		             "%s.%d (%s): device type unknown",
 		             __FILE__, __LINE__, __func__);
 		goto error;
 	}
 
 	if (!dclass->get_secrets) {
-		g_set_error (&error, NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_SECRETS_UNAVAILABLE,
+		g_set_error (&error,
+		             NM_SETTINGS_INTERFACE_ERROR,
+		             NM_SETTINGS_INTERFACE_ERROR_SECRETS_UNAVAILABLE,
 		             "%s.%d (%s): no secrets found",
 		             __FILE__, __LINE__, __func__);
 		goto error;
 	}
 
-	/* Let the device class handle secrets */
-	if (!dclass->get_secrets (device, connection, active_connection, setting_name,
-	                          hints, context, applet, &error))
-		goto error;
+	// FIXME: get secrets locally and populate connection with previous secrets
+	// before asking user for other secrets
 
-	return;
+	/* Let the device class handle secrets */
+	if (dclass->get_secrets (device, NM_SETTINGS_CONNECTION_INTERFACE (connection),
+	                         active_connection, setting_name, hints, callback,
+	                         callback_data, applet, &error))
+		return;  /* success */
 
 error:
 	g_warning ("%s", error->message);
-	dbus_g_method_return_error (context, error);
+	callback (NM_SETTINGS_CONNECTION_INTERFACE (connection), NULL, error, callback_data);
 	g_error_free (error);
 }
 
@@ -2283,8 +2611,7 @@ periodic_update_active_connection_timestamps (gpointer user_data)
 	for (i = 0; connections && (i < connections->len); i++) {
 		NMActiveConnection *active = NM_ACTIVE_CONNECTION (g_ptr_array_index (connections, i));
 		const char *path;
-		NMAGConfConnection *gconf_connection;
-		NMConnection *connection;
+		NMSettingsConnectionInterface *connection;
 		const GPtrArray *devices;
 		int k;
 
@@ -2292,8 +2619,8 @@ periodic_update_active_connection_timestamps (gpointer user_data)
 			continue;
 
 		path = nm_active_connection_get_connection (active);
-		gconf_connection = nma_gconf_settings_get_by_dbus_path (applet->gconf_settings, path);
-		if (!gconf_connection)
+		connection = nm_settings_interface_get_connection_by_path (NM_SETTINGS_INTERFACE (applet->gconf_settings), path);
+		if (!connection)
 			continue;
 
 		devices = nm_active_connection_get_devices (active);
@@ -2309,12 +2636,11 @@ periodic_update_active_connection_timestamps (gpointer user_data)
 			if (nm_device_get_state (device) == NM_DEVICE_STATE_ACTIVATED) {
 				NMSettingConnection *s_con;
 
-				connection = nm_exported_connection_get_connection (NM_EXPORTED_CONNECTION (gconf_connection));
-				s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+				s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (NM_CONNECTION (connection), NM_TYPE_SETTING_CONNECTION));
 				g_assert (s_con);
 
 				g_object_set (s_con, NM_SETTING_CONNECTION_TIMESTAMP, (guint64) time (NULL), NULL);
-				nma_gconf_connection_save (gconf_connection);
+				nm_settings_connection_interface_update (connection, save_timestamp_cb, NULL);
 				break;
 			}
 		}
@@ -2351,6 +2677,7 @@ static void nma_icons_free (NMApplet *applet)
 	CLEAR_ICON(applet->wireless_50_icon);
 	CLEAR_ICON(applet->wireless_75_icon);
 	CLEAR_ICON(applet->wireless_100_icon);
+	CLEAR_ICON(applet->secure_lock_icon);
 
 	for (i = 0; i < NUM_CONNECTING_STAGES; i++) {
 		for (j = 0; j < NUM_CONNECTING_FRAMES; j++)
@@ -2368,7 +2695,7 @@ static void nma_icons_free (NMApplet *applet)
 
 #define ICON_LOAD(icon, name) \
 	{ \
-		icon = gtk_icon_theme_load_icon (applet->icon_theme, name, applet->size, 0, &err); \
+		icon = gtk_icon_theme_load_icon (applet->icon_theme, name, applet->icon_size, 0, &err); \
 		if (icon == NULL) { \
 			g_warning ("Icon %s missing: %s", name, \
 			           (err && err->message) ? err->message : "unknown"); \
@@ -2384,9 +2711,7 @@ nma_icons_load (NMApplet *applet)
 	GError *err = NULL;
 
 	g_return_val_if_fail (!applet->icons_loaded, FALSE);
-
-	if (applet->size < 0)
-		return FALSE;
+	g_return_val_if_fail (applet->icon_size > 0, FALSE);
 
 	ICON_LOAD(applet->no_connection_icon, "nm-no-connection");
 	ICON_LOAD(applet->wired_icon, "nm-device-wired");
@@ -2399,6 +2724,7 @@ nma_icons_load (NMApplet *applet)
 	ICON_LOAD(applet->wireless_50_icon, "nm-signal-50");
 	ICON_LOAD(applet->wireless_75_icon, "nm-signal-75");
 	ICON_LOAD(applet->wireless_100_icon, "nm-signal-100");
+	ICON_LOAD(applet->secure_lock_icon, "nm-secure-lock");
 
 	for (i = 0; i < NUM_CONNECTING_STAGES; i++) {
 		for (j = 0; j < NUM_CONNECTING_FRAMES; j++) {
@@ -2485,7 +2811,10 @@ status_icon_size_changed_cb (GtkStatusIcon *icon,
                              gint size,
                              NMApplet *applet)
 {
-	applet->size = size;
+	/* icon_size may be 0 if for example the panel hasn't given us any space
+	 * yet.  We'll get resized later, but for now just load the 16x16 icons.
+	 */
+	applet->icon_size = MAX (16, size);
 
 	nma_icons_free (applet);
 	nma_icons_load (applet);
@@ -2551,9 +2880,6 @@ setup_widgets (NMApplet *applet)
 	applet->context_menu = nma_context_menu_create (applet);
 	if (!applet->context_menu)
 		return FALSE;
-	applet->encryption_size_group = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
-	if (!applet->encryption_size_group)
-		return FALSE;
 
 	return TRUE;
 }
@@ -2664,13 +2990,15 @@ constructor (GType type,
 	}
 	g_signal_connect (G_OBJECT (dbus_mgr), "exit-now", G_CALLBACK (exit_cb), applet);
 
-	applet->dbus_settings = (NMDBusSettings *) nm_dbus_settings_system_new (applet_dbus_manager_get_connection (dbus_mgr));
+	applet->system_settings = nm_remote_settings_system_new (applet_dbus_manager_get_connection (dbus_mgr));
 
 	applet->gconf_settings = nma_gconf_settings_new (applet_dbus_manager_get_connection (dbus_mgr));
 	g_signal_connect (applet->gconf_settings, "new-secrets-requested",
 	                  G_CALLBACK (applet_settings_new_secrets_requested_cb),
 	                  applet);
 
+	nm_settings_service_export (NM_SETTINGS_SERVICE (applet->gconf_settings));
+
 	/* Start our DBus service */
 	if (!applet_dbus_manager_start_service (dbus_mgr)) {
 		g_object_unref (applet);
@@ -2690,6 +3018,9 @@ constructor (GType type,
 	applet->cdma_class = applet_device_cdma_get_class (applet);
 	g_assert (applet->cdma_class);
 
+	applet->bt_class = applet_device_bt_get_class (applet);
+	g_assert (applet->bt_class);
+
 	foo_client_setup (applet);
 
 	/* timeout to update connection timestamps every 5 minutes */
@@ -2738,9 +3069,6 @@ static void finalize (GObject *object)
 	if (applet->gconf_client)
 		g_object_unref (applet->gconf_client);
 
-	if (applet->encryption_size_group)
-		g_object_unref (applet->encryption_size_group);
-
 	if (applet->status_icon)
 		g_object_unref (applet->status_icon);
 
@@ -2751,9 +3079,9 @@ static void finalize (GObject *object)
 		g_object_unref (applet->gconf_settings);
 		applet->gconf_settings = NULL;
 	}
-	if (applet->dbus_settings) {
-		g_object_unref (applet->dbus_settings);
-		applet->dbus_settings = NULL;
+	if (applet->system_settings) {
+		g_object_unref (applet->system_settings);
+		applet->system_settings = NULL;
 	}
 
 	G_OBJECT_CLASS (nma_parent_class)->finalize (object);
@@ -2765,7 +3093,7 @@ static void nma_init (NMApplet *applet)
 	applet->animation_step = 0;
 	applet->icon_theme = NULL;
 	applet->notification = NULL;
-	applet->size = -1;
+	applet->icon_size = 16;
 }
 
 enum {
